
--#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#LOAD#
Ball = class(function(b, position, velocity, radius, color, density)
	position = position or Vector2(0,0);
	velocity = velocity or Vector2(0,0);
	radius = radius or 10;
	color = color or Colors.white;
	density = density or 1;
	
	b.position = position;
	b.velocity = velocity;
	b.radius = radius;
	b.color = color;
	b.density = density;
end)

--#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#
function Ball:Update(scalar)
	--Update with respect to velocity
	if (not self.velocity:isZero()) then
		self.position = self.position + self.velocity * scalar;
	end
	
	--Bounce off walls
	if (self.position.x - self.radius < 0) then
		self.position.x = self.radius;
		self.velocity.x = math.abs(self.velocity.x);
	elseif (self.position.x + self.radius > screenSize.x ) then
		self.position.x = screenSize.x - self.radius;
		self.velocity.x = -math.abs(self.velocity.x);
	end
	if (self.position.y - self.radius < 0) then
		self.position.y = self.radius;
		self.velocity.y = math.abs(self.velocity.y);
	elseif (self.position.y + self.radius > screenSize.y ) then
		self.position.y = screenSize.y - self.radius;
		self.velocity.y = -math.abs(self.velocity.y);
	end
end

--#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#DRAW#
function Ball:Draw(image)
	love.graphics.setColor(self.color);
	
	if (image) then
		love.graphics.draw(image, self.position.x, self.position.y, 0,
			(radius * 2) / image:getWidth(), (radius * 2) / image:getHeight(),
			image:getWidth() / 2, image:getHeight() / 2);
	else
		love.graphics.circle("fill", self.position.x, self.position.y, self.radius);
	end
end

--#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#OTHER#

function Ball:resolveCollision(ball)
	local delta = self.position - ball.position;
	local d = delta:length();
	--minimum translation distance to push balls apart from intersecting
	local mtd = delta * (((self.radius + ball.radius) - d) / d);
	
	--some helpful variables that only show up in code
	local restitution = 1;
	local mass1 = math.pi * math.pow(self.radius, 2) * self.density;
	local mass2 = math.pi * math.pow(ball.radius, 2) * ball.density;
	
	--resolve intersection--
	--inverse mass quantities
	local im1 = 1 / mass1;
	local im2 = 1 / mass2;
	
	--push/pull them apart based off their mass
	self.position:add(mtd * im1 / (im1 + im2));
	ball.position:sub(mtd * im2 / (im1 + im2));
	
	--impact speed
	local v = self.velocity - ball.velocity;
	if (not mtd:isZero()) then
		mtd = mtd / mtd:length();
	end
	local vn = dotVector2(v, mtd);
	
	--spheres intersecting but moving away from each other already
	if (vn > 0) then
		return;
	end
	
	--collision impulse
	local i = (-(1 + restitution) * vn) / (im1 + im2);
	local impulse = mtd * i;
	
	--change in momentum
	self.velocity:add(impulse * im1);
	ball.velocity:sub(impulse * im2);
end