Vector2 = class(
function(vec, x, y)
	vec.x = x;
	vec.y = y;
	
	local mt = getmetatable(vec);
	mt.__add=function(a,b)
		if (type(a) == "number") then
			return Vector2(b.x + a, b.y + a);
		elseif (type(b) == "number") then
			return Vector2(a.x + b, a.y + b);
		else
			return Vector2(a.x + b.x, a.y + b.y);
		end
	end
	mt.__sub=function(a,b)
		if (type(a) == "number") then
			return Vector2(a - b.x, a - b.y);
		elseif (type(b) == "number") then
			return Vector2(a.x - b, a.y - b);
		else
			return Vector2(a.x - b.x, a.y - b.y);
		end
	end
	mt.__mul=function(a,b)
		if (type(a) == "number") then
			return Vector2(b.x * a, b.y * a);
		elseif (type(b) == "number") then
			return Vector2(a.x * b, a.y * b);
		else
			return Vector2(a.x * b.x, a.y * b.y);
		end
	end
	mt.__div=function(a,b)
		if (type(a) == "number") then
			return Vector2(a / b.x, a / b.y);
		elseif (type(b) == "number") then
			return Vector2(a.x / b, a.y / b);
		else
			return Vector2(a.x / b.x, a.y / b.y);
		end
	end
	mt.__unm=function(a)
		return Vector2(-a.x, -a.y);
	end
	
	setmetatable(vec, mt);
end);

function Vector2:setX(value)
	self.x = value;
end
function Vector2:setY(value)
	self.y = value;
end

--[[#ARITHMETIC#ARITHMETIC#ARITHMETIC#ARITHMETIC#ARITHMETIC#ARITHMETIC#ARITHMETIC#
Includes:
Vector2:add(vec2)
Vector2:add2(addX, addY)
addVector2(vec1, vec2)

Vector2:negative()
negativeVector2(vec2)

Vector2:sub(vec2)
Vector2:sub2(subX, subY)
subVector2(vec1, vec2)

Vector2:mul(vec2)
Vector2:mul2(mulX, mulY)
mulVector2(vec1, vec2)

Vector2:div(vec2)
Vector2:div2(divX, divY)
divVector2(vec1, vec2)

Vector2:scale(scale)
Vector2:scale2(vec2)
scaleVector2(vec2, scale)
scale2Vector2(vec1, vec2)

Vector2:dot(vec2)
dotVector2(vec1, vec2)

Vector2:equals(vec2)
equalsVector2(vec1, vec2)

Vector2:distance(vec2)
distanceVector2(vec1, vec2)

Vector2:length()
lengthVector2(vec2)

Vector2:normalize()
normalizeVector2(vec2)

Vector2:isZero()
isZeroVector2(vec2)
]]

function Vector2:add(b)
	self.x = self.x + b.x;
	self.y = self.y + b.y;
end
function Vector2:add2(addX, addY)
	self.x = self.x + addX;
	self.y = self.y + addY;
end
function addVector2(vec1, vec2)
	return Vector2(vec1.x + vec2.x, vec1.y + vec2.y);
end

function Vector2:negative()
	self.x = -self.x;
	self.y = -self.y;
end
function negativeVector2(vec2)
	return Vector2(-vec2.x, -vec2.y);
end

function Vector2:sub(vec2)
	self.x = self.x - vec2.x;
	self.y = self.y - vec2.y;
end
function Vector2:sub2(subX, subY)
	self.x = self.x - subX;
	self.y = self.y - subY;
end
function subVector2(vec1, vec2)
	return Vector2(vec1.x - vec2.x, vec1.y - vec2.y);
end

function Vector2.mul(vec2)
	self.x = self.x * vec2.x;
	self.y = self.y * vec2.y;
end
function Vector2:mul2(mulX, mulY)
	self.x = self.x * mulX;
	self.y = self.y * mulY;
end
function mulVector2(vec1, vec2)
	return Vector2(vec1.x * vec2.x, vec1.y * vec2.y);
end

function Vector2.div(vec2)
	self.x = self.x / vec2.x;
	self.y = self.y / vec2.y;
end
function Vector2:div2(divX, divY)
	self.x = self.x / divX;
	self.y = self.y / divY;
end
function divVector2(vec1, vec2)
	return Vector2(vec1.x / vec2.x, vec1.y / vec2.y);
end

function Vector2:scale(scale)
	self.x = self.x * scale;
	self.y = self.y * scale;
end
function Vector2:scale2(vec2)
	self.x = self.x * vec2.x;
	self.y = self.y * vec2.y;
end
function scaleVector2(vec2, scale)
	return Vector2(vec2.x  * scale, vec2.y * scale);
end
function scale2Vector2(vec1, vec2)
	return Vector2(vec1.x  * vec2.x, vec1.y * vec2.y);
end

function Vector2:dot(vec2)
	return (self.x * vec2.x) + (self.y + vec2.y);
end
function dotVector2(vec1, vec2)
	return (vec1.x * vec2.x) + (vec1.y * vec2.y);
end

function Vector2:equals(vec2)
	return (self.x == vec2.x and self.y == vec2.y);
end
function equalsVector2(vec1, vec2)
	return (vec1.x == vec2.x and vec1.y == vec2.y);
end

function Vector2:distance(vec2)
	return math.sqrt(math.pow(vec2.x - self.x, 2) + math.pow(vec2.y - self.y, 2));
end
function distanceVector2(vec1, vec2)
	return math.sqrt(math.pow(vec2.x - vec1.x, 2) + math.pow(vec2.y - vec1.y, 2));
end

function Vector2:length()
	return math.sqrt(math.pow(self.x, 2) + math.pow(self.y, 2));
end
function lengthVector2(vec2)
	return math.sqrt(math.pow(vec2.x, 2) + math.pow(vec2.y, 2));
end

function Vector2:normalize()
	local len = lengthVector2(self);
	self = self / len;
end
function normalizeVector2(vec2)
	local len = lengthVector2(vec2);
	return vec2 / len;
end

function Vector2:isZero()
	return (self.x == 0 and self.y == 0);
end
function isZeroVector2(vec2)
	return (vec2.x == 0 and vec2.y == 0);
end